#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2021-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Console script for Trust provisioning host application."""
import logging
import os
import sys
from typing import List

import click

from spsdk.apps.tp_utils import (
    TPHostConfig,
    device_help,
    get_counters,
    list_tpdevices,
    list_tptargets,
    process_tp_inputs,
    target_help,
    tp_device_options,
    tp_target_options,
)
from spsdk.apps.utils import (
    INT,
    CommandsTreeGroupAliasedGetCfgTemplate,
    catch_spsdk_error,
    spsdk_apps_common_options,
    spsdk_logger,
)
from spsdk.apps.utils.utils import SPSDKAppError
from spsdk.crypto import Encoding, ec
from spsdk.crypto.loaders import extract_public_key, load_certificate_from_data
from spsdk.exceptions import SPSDKError
from spsdk.tp import TP_DATA_FOLDER, TpDevInterface, TpTargetInterface, TrustProvisioningHost
from spsdk.tp.data_container import Container
from spsdk.tp.data_container.payload_types import PayloadType
from spsdk.tp.exceptions import SPSDKTpError
from spsdk.tp.utils import (
    get_supported_devices,
    reconstruct_cryptography_key,
    scan_tp_devices,
    scan_tp_targets,
)
from spsdk.utils.misc import load_binary, load_text, write_file


@click.group(name="tphost", cls=CommandsTreeGroupAliasedGetCfgTemplate)
@spsdk_apps_common_options
def main(log_level: int) -> int:
    """Application to secure Trust provisioning process of loading application in Un-trusted environment."""
    spsdk_logger.install(level=log_level)
    return 0


@main.command(name="load", no_args_is_help=True)
@tp_device_options
@tp_target_options
@click.option(
    "-f",
    "--family",
    type=click.Choice(get_supported_devices(), case_sensitive=False),
    help="The target device name.",
)
@click.option(
    "-fw",
    "--firmware",
    type=click.Path(exists=True, dir_okay=False),
    help="The application firmware SB file. If not specified, TP flow ends after loading OEM assets and reset.",
)
@click.option(
    "-pfw",
    "--prov-firmware",
    type=click.Path(exists=True, dir_okay=False),
    help="OEM Provisioning Firmware SB file. If not specified, the TP flow starts immediately.",
)
@click.option(
    "-to",
    "--timeout",
    type=click.IntRange(0, 600, clamp=True),
    help="The target provisioning timeout in seconds.",
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False),
    help="Path to TPHost config file.",
)
@click.option(
    "-l",
    "--audit-log",
    type=click.Path(exists=False, dir_okay=False),
    help="Path TP audit log yaml file.",
)
@click.option(
    "-s",
    "--save-debug-data",
    is_flag=True,
    default=False,
    help="Save the data being transferred (for debugging purposes).",
)
def load(
    tp_device: str,
    tp_device_parameter: List[str],
    tp_target: str,
    tp_target_parameter: List[str],
    family: str,
    firmware: str,
    prov_firmware: str,
    timeout: int,
    config: str,
    audit_log: str,
    save_debug_data: bool,
) -> None:
    """Command to provision target MCU."""
    tp_config = TPHostConfig(
        tp_device=tp_device,
        tp_device_parameter=tp_device_parameter,
        tp_target=tp_target,
        tp_target_parameter=tp_target_parameter,
        family=family,
        firmware=firmware,
        prov_firmware=prov_firmware,
        audit_log=audit_log,
        timeout=timeout,
        config=config,
    )

    tp_interface = process_tp_inputs(
        tp_type=tp_config.tp_device,
        tp_parameters=tp_config.tp_device_parameter,
        header="device",
        scan_func=scan_tp_devices,
        print_func=click.echo,
    )
    tp_device_instance = tp_interface.create_interface()
    assert isinstance(tp_device_instance, TpDevInterface)

    tp_interface = process_tp_inputs(
        tp_type=tp_config.tp_target,
        tp_parameters=tp_config.tp_target_parameter,
        header="target",
        scan_func=scan_tp_targets,
        print_func=click.echo,
    )
    tp_target_instance = tp_interface.create_interface()
    assert isinstance(tp_target_instance, TpTargetInterface)

    tp_worker = TrustProvisioningHost(tp_device_instance, tp_target_instance, click.echo)
    tp_worker.do_provisioning(
        family=tp_config.family,
        audit_log=tp_config.audit_log,
        prov_fw=tp_config.prov_firmware_data,
        product_fw=tp_config.firmware_data,
        timeout=tp_config.timeout,
        save_debug_data=save_debug_data,
    )


@main.command(name="load-tpfw", no_args_is_help=True)
@tp_target_options
@click.option(
    "-f",
    "--family",
    type=click.Choice(get_supported_devices(), case_sensitive=False),
    help="The target device name.",
)
@click.option(
    "-pfw",
    "--prov-firmware",
    type=click.Path(exists=True, dir_okay=False),
    help="OEM Provisioning Firmware SB file.",
)
@click.option(
    "-to",
    "--timeout",
    type=click.IntRange(0, 600, clamp=True),
    help="The target provisioning timeout in seconds.",
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False),
    help="Path to TPHost config file.",
)
@click.option(
    "-s",
    "--skip-test",
    is_flag=True,
    help="Skip test whether Provisioning FW boot-ed up.",
)
def load_tpfw(
    tp_target: str,
    tp_target_parameter: List[str],
    family: str,
    prov_firmware: str,
    timeout: int,
    config: str,
    skip_test: bool,
) -> None:
    """Command to load Provisioning Firmware to the target MCU."""
    TPHostConfig.SCHEMA_MEMBERS = ["family", "provisioning_firmware", "tp_timeout", "target"]
    tp_config = TPHostConfig(
        tp_target=tp_target,
        tp_target_parameter=tp_target_parameter,
        family=family,
        prov_firmware=prov_firmware,
        timeout=timeout,
        config=config,
    )

    if not tp_config.prov_firmware_data:
        raise SPSDKAppError("Provisioning Firmware to load is not defined.")

    tp_interface = process_tp_inputs(
        tp_type=tp_config.tp_target,
        tp_parameters=tp_config.tp_target_parameter,
        header="target",
        scan_func=scan_tp_targets,
        print_func=click.echo,
    )
    tp_target_instance = tp_interface.create_interface()
    assert isinstance(tp_target_instance, TpTargetInterface)

    tp_worker = TrustProvisioningHost(
        tpdev=None,  # type: ignore  # we don't need TPDevice in this case
        tptarget=tp_target_instance,
        info_print=click.echo,
    )
    tp_worker.load_provisioning_fw(
        family=tp_config.family,
        prov_fw=tp_config.prov_firmware_data,
        timeout=tp_config.timeout,
        skip_test=skip_test,
        keep_target_open=False,
        skip_usb_enumeration=skip_test,
    )


@main.command(name="get-template", no_args_is_help=True)
@click.option(
    "-f",
    "--family",
    type=click.Choice(get_supported_devices(), case_sensitive=False),
    default="lpc55s6x",
    help="Chip family to generate the TPHost config for.",
)
@click.option(
    "-o",
    "--output",
    type=click.Path(dir_okay=False, resolve_path=True),
    required=True,
    help="The output YAML template configuration file name.",
)
# pylint: disable=unused-argument   # preparation for the future
def get_template(
    family: str,
    output: str,
) -> None:
    """Command to generate tphost template of configuration YML file."""
    template = load_text(os.path.join(TP_DATA_FOLDER, "tphost_cfg_template.yaml"))
    write_file(template, output)

    click.echo(f"The configuration template created. {output}")


@main.command(name="verify", no_args_is_help=True)
@click.option(
    "-l",
    "--audit-log",
    type=click.Path(exists=True, dir_okay=False),
    required=True,
    help="Path TP audit log yaml file.",
)
@click.option(
    "-k",
    "--audit-log-key",
    type=click.Path(exists=True, dir_okay=False),
    required=True,
    help="Path to private/public key to verify the TP audit log yaml file.",
)
@click.option(
    "-d",
    "--destination",
    type=click.Path(file_okay=False),
    help="Destination directory for certificate extraction(non-existent or empty).",
)
@click.option(
    "-e",
    "--encoding",
    type=click.Choice(["PEM", "DER"], case_sensitive=False),
    default="PEM",
    show_default=True,
    help="X509 certificate encoding.",
)
@click.option(
    "-n",
    "--skip-nxp",
    is_flag=True,
    default=False,
    help="Skip extracting the NXP Devattest certificates.",
)
@click.option(
    "-o",
    "--skip-oem",
    is_flag=True,
    default=False,
    help="Skip extracting the OEM x509 Devattest certificates.",
)
@click.option(
    "-i",
    "--cert-index",
    type=click.IntRange(0, 3),
    show_choices=True,
    metavar="[0-3]",
    help="""
    Index of an individual OEM certificate to extract.
    If not specified, all available OEM certificates will be extracted.
    """,
)
@click.option(
    "-j",
    "--processes",
    type=INT(),
    help=f"How many processes to use; if not specified use cpu_count: {os.cpu_count()}",
)
@click.option(
    "--force-rewrite",
    is_flag=True,
    default=False,
    help="Rewrite certificates in 'destination' directory.",
)
def verify(
    audit_log: str,
    audit_log_key: str,
    destination: str,
    encoding: str,
    skip_nxp: bool,
    skip_oem: bool,
    cert_index: int,
    processes: int,
    force_rewrite: bool,
) -> None:
    """Verify audit log integrity and optionally extract certificates.

    Certificate extraction takes place if `-d/--destination` is specified.
    """
    TrustProvisioningHost.verify_extract_log(
        audit_log=audit_log,
        audit_log_key=audit_log_key,
        destination=destination,
        skip_nxp=skip_nxp,
        skip_oem=skip_oem,
        cert_index=cert_index,
        encoding=Encoding.PEM if encoding.lower() == "pem" else Encoding.DER,
        max_processes=processes,
        info_print=click.echo,
        force_rewrite=force_rewrite,
    )


@main.command(name="check-log-owner", no_args_is_help=True)
@tp_device_options
@click.option(
    "-t",
    "--timeout",
    type=click.IntRange(0, 600, clamp=True),
    help="The target provisioning timeout in seconds.",
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False),
    help="Path to configuration file (parameters on CLI take precedence).",
    required=False,
)
@click.option(
    "-l",
    "--audit-log",
    type=click.Path(exists=True, dir_okay=False),
    required=False,
    help="Path TP audit log yaml file.",
)
def check_log_owner(
    tp_device: str,
    tp_device_parameter: List[str],
    timeout: int,
    config: str,
    audit_log: str,
) -> None:
    """Check whether TP Device ID matches the ID in audit log."""
    TPHostConfig.SCHEMA_MEMBERS = ["device", "audit_log"]
    tp_config = TPHostConfig(
        tp_device=tp_device,
        tp_device_parameter=tp_device_parameter,
        config=config,
        timeout=timeout,
        audit_log=audit_log,
    )

    tp_interface = process_tp_inputs(
        tp_type=tp_config.tp_device,
        tp_parameters=tp_config.tp_device_parameter,
        header="device",
        scan_func=scan_tp_devices,
        print_func=click.echo,
    )
    tp_dev = tp_interface.create_interface()
    assert isinstance(tp_dev, TpDevInterface)

    tp_worker = TrustProvisioningHost(
        tpdev=tp_dev,
        tptarget=None,  # type: ignore  # target is not used, we set it to None on purpose
        info_print=click.echo,
    )
    tp_worker.check_audit_log_owner(tp_config.audit_log, timeout=tp_config.timeout)


@main.command(name="get-tp-response", no_args_is_help=True)
@tp_target_options
@click.option(
    "-f",
    "--family",
    type=click.Choice(get_supported_devices(), case_sensitive=False),
    help="The target device name.",
)
@click.option(
    "-to",
    "--timeout",
    type=click.IntRange(0, 600, clamp=True),
    help="The target provisioning timeout in seconds.",
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False),
    help="Path to TPHost configuration file (parameters on CLI take precedence).",
    required=False,
)
@click.option(
    "-r",
    "--response-file",
    type=click.Path(dir_okay=False),
    help="Path where to store the TP_RESPONSE",
    required=True,
)
def get_tp_response(
    tp_target: str,
    tp_target_parameter: List[str],
    family: str,
    timeout: int,
    config: str,
    response_file: str,
) -> None:
    """Retrieve TP_RESPONSE from the target."""
    TPHostConfig.SCHEMA_MEMBERS = ["family", "tp_timeout", "target"]

    tp_config = TPHostConfig(
        tp_target=tp_target,
        tp_target_parameter=tp_target_parameter,
        family=family,
        timeout=timeout,
        config=config,
    )

    tp_interface = process_tp_inputs(
        tp_type=tp_config.tp_target,
        tp_parameters=tp_config.tp_target_parameter,
        header="target",
        scan_func=scan_tp_targets,
        print_func=click.echo,
    )
    tp_target_instance = tp_interface.create_interface()
    assert isinstance(tp_target_instance, TpTargetInterface)

    tp_worker = TrustProvisioningHost(
        tpdev=None,  # type: ignore  # device is not used, we set it to None on purpose
        tptarget=tp_target_instance,
        info_print=click.echo,
    )
    tp_worker.get_tp_response(
        response_file=response_file,
        timeout=tp_config.timeout,
    )


@main.command(name="check-cot", no_args_is_help=True)
@click.option(
    "-r",
    "--root-cert",
    help="NXP Glob Root Certificate authority certificate.",
    type=click.Path(exists=True, dir_okay=False),
    required=False,
)
@click.option(
    "-i",
    "--intermediate-cert",
    help="NXP Product Intermediate certificate.",
    type=click.Path(exists=True, dir_okay=False),
    required=True,
)
@click.option(
    "-t",
    "--tp-response",
    help="TP Response from MCU.",
    type=click.Path(exists=True, dir_okay=False),
    required=True,
)
# pylint: disable=broad-except  # true source of potential error is not known in advance
def check_cot(root_cert: str, intermediate_cert: str, tp_response: str) -> None:
    """Check Chain-of-Trust in Trust Provisioning.

    \b
    Root and intermediate certificates are provided from NXP.
    TP_RESPONSE can be obtained from `tphost load` using `--save-debug-data` flag.
    Default file name for TP_RESPONSE is `x_tp_response.bin`.
    """
    overall_result = True

    nxp_glob_puk = extract_public_key(root_cert) if root_cert else None
    if not nxp_glob_puk:
        click.echo("NXP_GLOB key/cert not specified. NXP_PROD cert verification will be skipped.")

    nxp_prod_cert_data = load_binary(intermediate_cert)
    try:
        nxp_prod_cert = load_certificate_from_data(nxp_prod_cert_data)
        nxp_prod_puk = nxp_prod_cert.public_key()
    except SPSDKError as e:
        logging.debug(str(e))
        if nxp_glob_puk:
            raise SPSDKAppError(f"Unable to load NXP_PROD certificate: {str(e)}") from e
        click.echo("Failed to load NXP_PROD as certificate, attempting to load raw public key")
        nxp_prod_puk = reconstruct_cryptography_key(key_material=nxp_prod_cert_data)
    assert isinstance(nxp_prod_puk, ec.EllipticCurvePublicKey)

    if nxp_glob_puk:
        assert isinstance(nxp_glob_puk, ec.EllipticCurvePublicKey)
        message = "validating NXP_PROD_CERT signature..."
        try:
            assert nxp_prod_cert.signature_hash_algorithm
            nxp_glob_puk.verify(
                nxp_prod_cert.signature,
                nxp_prod_cert.tbs_certificate_bytes,
                ec.ECDSA(nxp_prod_cert.signature_hash_algorithm),
            )
            message += "OK"
        except Exception:
            message += "FAILED!"
            overall_result = False
        click.echo(message)

    message = "Parsing TP_RESPONSE data container..."
    try:
        tp_data = load_binary(tp_response)
        tp_response_container = Container.parse(tp_data)
        message += "OK"
    except Exception:
        message += "FAILED!"
        overall_result = False
    click.echo(message)

    message = "Extracting NXP_DIE_ID_Devattest_CERT..."
    try:
        nxp_die_cert_data = tp_response_container.get_entry(
            PayloadType.NXP_DIE_ID_AUTH_CERT
        ).payload
        nxp_die_cert = Container.parse(nxp_die_cert_data)
        message += "OK"
    except Exception:
        message += "FAILED!"
        overall_result = False
    click.echo(message)

    assert nxp_die_cert
    message = "Validating NXP_DIE_ID_Devattest_CERT signature..."
    if nxp_die_cert.validate(nxp_prod_puk):  # type: ignore
        message += "OK"
    else:
        message += "FAILED!"
        overall_result = False
    click.echo(message)

    message = "Validating TP_RESPONSE signature..."
    try:
        nxp_die_puk_data = nxp_die_cert.get_entry(PayloadType.NXP_DIE_ATTEST_AUTH_PUK).payload
    except SPSDKTpError:
        nxp_die_puk_data = nxp_die_cert.get_entry(PayloadType.NXP_DIE_ID_AUTH_PUK).payload
    nxp_die_puk = reconstruct_cryptography_key(nxp_die_puk_data)
    if tp_response_container.validate(nxp_die_puk):  # type: ignore
        message += "OK"
    else:
        message += "FAILED!"
        overall_result = False
    click.echo(message)
    if overall_result is False:
        raise SPSDKAppError()


main.add_command(device_help)
main.add_command(target_help)
main.add_command(list_tpdevices)
main.add_command(list_tptargets)
main.add_command(get_counters)


@catch_spsdk_error
def safe_main() -> None:
    """Call the main function."""
    sys.exit(main())  # pragma: no cover  # pylint: disable=no-value-for-parameter


if __name__ == "__main__":
    safe_main()  # pragma: no cover



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2022-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause
"""DK6 UART communication interface."""
import logging
import struct
from time import sleep
from typing import Any, Union

from crcmod.predefined import mkPredefinedCrcFun

from spsdk import SPSDKError

from .commands import CmdPacket, CommandTag, parse_cmd_response
from .serial_device import SerialDevice

logger = logging.getLogger(__name__)


def calc_crc(data: bytes) -> int:
    """Calculate CRC from the data.

    :param data: data to calculate CRC from
    :return: calculated CRC
    """
    crc_function = mkPredefinedCrcFun("crc-32")
    return crc_function(data)


def to_int(data: bytes, little_endian: bool = False) -> int:
    """Convert bytes into single integer.

    :param data: bytes to convert
    :param little_endian: indicate byte ordering in data, defaults to True
    :return: integer
    """
    byte_order = "little" if little_endian else "big"
    return int.from_bytes(data, byteorder=byte_order)  # type: ignore[arg-type]


class Uart:
    """UART interface for DK6 devices."""

    FRAME_START_BYTE = 0x00

    FLAG_SIZE = 1
    LENGTH_SIZE = 2
    FRAME_TYPE_SIZE = 1
    CHECKSUM_SIZE = 4

    HEADER_SIZE = FLAG_SIZE + LENGTH_SIZE + FRAME_TYPE_SIZE + CHECKSUM_SIZE

    @property
    def is_opened(self) -> bool:
        """Return True if device is open, False otherwise."""
        return self.device.is_opened

    def __init__(
        self,
        device: SerialDevice,
    ) -> None:
        """Initialize the UART interface.

        :param port: name of the serial port, defaults to None
        :raises McuBootConnectionError: when the port could not be opened
        """
        self.device = device
        # self.device.baudrate = baudrate

    def open(self) -> None:
        """Open the UART interface.

        :raises SPSDKError: In any case of fail of UART open operation.
        """
        try:
            self.device.open()
        except Exception as exc:
            raise SPSDKError(f"Cannot open UART interface: {exc}") from exc

    def close(self) -> None:
        """Close the UART interface.

        :raises SPSDKError: In any case of fail of UART close operation.
        """
        try:
            self.device.close()
        except Exception as exc:
            raise SPSDKError(f"Cannot close UART interface: {exc}") from exc

    def read(self) -> Any:
        """Read data from device.

        :return: CmdResponse
        :raises SPSDKError: Did not receive correct frame start byte
        :raises SPSDKError: When received invalid CRC
        """
        flag = to_int(self._read_default(self.FLAG_SIZE))
        if flag != self.FRAME_START_BYTE:
            raise SPSDKError("Did not receive correct frame start byte")

        length = to_int(self._read_default(self.LENGTH_SIZE))
        frame_type = to_int(self._read_default(self.FRAME_TYPE_SIZE))

        data = self._read_default(length - self.HEADER_SIZE)
        crc = to_int(self._read_default(self.CHECKSUM_SIZE))

        calculated_crc = self.calc_frame_crc(data, frame_type)
        if crc != calculated_crc:
            raise SPSDKError("Received invalid CRC")

        logger.debug(
            f"<-READ flag: {hex(flag)}, length: {hex(length)}, "
            f"frame_type: {hex(frame_type)}, data: <{' '.join(f'{b:02x}' for b in data)}>,"
            f" crc: {hex(crc)}"
        )
        return parse_cmd_response(data, frame_type)

    def write(self, frame_type: CommandTag, packet: Union[CmdPacket, bytes, None]) -> None:
        """Write data to the device; data might be in form of 'CmdPacket' or bytes.

        :param frame_type: CommandTag
        :param packet: Packet to send
        :raises AttributeError: frame type is incorrect
        """
        if isinstance(packet, CmdPacket):
            packet = packet.to_bytes()

        frame = self.create_frame(packet, frame_type)
        # logger.debug(f"Packet {packet}, frame type; {frame_type}")
        # logger.debug(f"->WRITE: frame: {frame}")
        self._send_frame(frame)

    def _read_default(self, length: int) -> bytes:
        """Read 'length' amount of bytes from device, this function can be overridden in child class.

        :param length: Number of bytes to read
        :type length: int
        :raises SPSDKError: When read fails
        :return: Data read from the device
        """
        return self._read(length)

    def _read(self, length: int) -> bytes:
        """Read 'length' amount for bytes from device.

        :param length: Number of bytes to read
        :return: Data read from the device
        :raises TimeoutError: Time-out
        :raises SPSDKError: When reading data from device fails
        """
        try:
            data = self.device.read(length)
        except Exception as e:
            raise SPSDKError(str(e)) from e
        if not data:
            raise TimeoutError()
        logger.debug(f"<-READ:  <{' '.join(f'{b:02x}' for b in data)}>")
        return data

    def _write(self, data: bytes) -> None:
        """Send data to device.

        :param data: Data to send
        :raises SPSDKError: When sending the data fails
        """
        logger.debug(f"->WRITE: [{' '.join(f'{b:02x}' for b in data)}]")
        sleep(0.01)  # Sleep 10 ms before writing to prevent buffer overrun
        try:
            self.device.write(data)
        except Exception as e:
            raise SPSDKError(str(e)) from e

    def _send_frame(self, frame: bytes) -> None:
        """Send a frame to UART.

        :param frame: Data to send
        :param wait_for_ack: Wait for ACK frame from device, defaults to True
        """
        self._write(frame)

    @staticmethod
    def create_frame(data: Union[bytes, None], frame_type: Union[int, CommandTag]) -> bytes:
        """Encapsulate data into frame.

        :param data: payload data
        :param frame_type: frame type
        :return: frame
        """
        crc = Uart.calc_frame_crc(data, frame_type)
        if data:
            # print(f"Data length {len(data) + Uart.HEADER_SIZE}")
            frame = struct.pack(
                f">BHB{len(data)}BI",
                Uart.FRAME_START_BYTE,
                len(data) + Uart.HEADER_SIZE,
                frame_type,
                *data,
                crc,
            )
        else:
            frame = struct.pack(
                ">BHBI",
                Uart.FRAME_START_BYTE,
                Uart.HEADER_SIZE,
                frame_type,
                crc,
            )

        return frame

    @staticmethod
    def calc_frame_crc(data: Union[bytes, None], frame_type: Union[int, CommandTag]) -> int:
        """Calculate the CRC of a frame.

        :param data: frame data
        :param frame_type: frame type
        :return: calculated CRC
        """
        if data:
            crc_data = struct.pack(
                f">BHB{len(data)}B",
                Uart.FRAME_START_BYTE,
                len(data) + Uart.HEADER_SIZE,
                frame_type,
                *data,
            )
        else:
            crc_data = struct.pack(
                ">BHB",
                Uart.FRAME_START_BYTE,
                Uart.HEADER_SIZE,
                frame_type,
            )
        return calc_crc(crc_data)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2020-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause
"""The shadow registers control DAT support file."""

import logging
from typing import Any, List, Optional

from ruamel.yaml import YAML
from ruamel.yaml.comments import CommentedMap as CM

from spsdk import SPSDK_YML_INDENT, SPSDKError, __author__, __release__, __version__
from spsdk.dat.debug_mailbox import DebugMailbox
from spsdk.dat.dm_commands import StartDebugSession
from spsdk.debuggers.debug_probe import DebugProbe, SPSDKDebugProbeError
from spsdk.debuggers.utils import test_ahb_access
from spsdk.utils.misc import (
    change_endianness,
    load_configuration,
    value_to_bytes,
    value_to_int,
    write_file,
)
from spsdk.utils.reg_config import RegConfig
from spsdk.utils.registers import Registers, RegsRegister, SPSDKRegsErrorRegisterNotFound
from spsdk.utils.schema_validator import ConfigTemplate

logger = logging.getLogger(__name__)


class IoVerificationError(SPSDKError):
    """The error during write verification - exception for use with SPSDK."""


class ShadowRegisters:
    """SPSDK support to control the shadow registers."""

    def __init__(
        self,
        debug_probe: Optional[DebugProbe],
        config: RegConfig,
        device: str,
        revision: str = "latest",
    ) -> None:
        """Initialization of Shadow register class."""
        self.probe = debug_probe
        self.config = config
        self.device = device
        self.offset = value_to_int(self.config.get_address(self.device))
        self.fuse_mode = False

        self.regs = Registers(self.device)
        rev = revision or "latest"
        self.revision = (
            rev
            if rev != "latest"
            else config.devices.get_by_name(self.device).revisions.get_latest().name
        )
        self.regs.load_registers_from_xml(
            config.get_data_file(self.device, self.revision),
            grouped_regs=config.get_grouped_registers(self.device),
        )

        # Set the computed field handler
        for reg, fields in self.config.get_computed_fields(self.device).items():
            reg_obj = self.regs.find_reg(reg)
            reg_obj.add_setvalue_hook(self.reg_computed_fields_handler, fields)

        # Set the antipolize handler
        for reg, antipole_reg in self.config.get_antipole_regs(self.device).items():
            src = self.regs.find_reg(reg)
            dst = self.regs.find_reg(antipole_reg)
            src.add_setvalue_hook(self.reg_antipolize_src_handler, dst)
            dst.add_setvalue_hook(self.reg_antipolize_dst_handler, src)

    def _write_shadow_reg(self, addr: int, data: int, verify_mask: int = 0) -> None:
        """The function write a shadow register.

        The function writes shadow register in to MCU and verify the write if requested.

        param addr: Shadow register address.
        param data: Shadow register data to write.
        param verify_mask: Verify bit mask for read back and compare, if 0 verify is disable
        raises IoVerificationError
        """
        if not self.probe:
            raise SPSDKDebugProbeError(
                "Shadow registers: Cannot use the communication function without defined debug probe."
            )

        logger.debug(f"Writing shadow register address: {hex(addr)}, data: {hex(data)}")
        self.probe.mem_reg_write(addr, data)

        if verify_mask:
            read_back = self.probe.mem_reg_read(addr)
            if read_back & verify_mask != data & verify_mask:
                raise IoVerificationError(
                    f"The verification of written shadow register 0x{addr:08X} failed."
                    " Maybe a READ LOCK is set for that register."
                )

    def reload_registers(self) -> None:
        """Reload all the values in managed registers."""
        for reg in self.regs.get_registers():
            self.reload_register(reg)

    def sets_all_registers(self, verify: bool = True) -> None:
        """Update all shadow registers in target by local values.

        :param verify: Verity write operation.
        """
        for reg in self.regs.get_registers():
            self.set_register(reg.name, reg.get_value(), verify)

    def reload_register(self, reg: RegsRegister) -> None:
        """Reload the value in requested register.

        :param reg: The register to reload from the HW.
        """
        reg.set_value(self.get_register(reg.name), raw=True)

    def read_register(self, reg: RegsRegister) -> bytes:
        """Read the value in requested register.

        :param reg: The register to read from the HW.
        """
        return bytes(self.get_register(reg.name))

    def set_register(self, reg_name: str, data: Any, verify: bool = True) -> None:
        """The function sets the value of the specified register.

        :param reg_name: The register name.
        :param data: The new data to be stored to shadow register.
        :param verify: Verity write operation.
        :raises SPSDKDebugProbeError: The debug probe is not specified.
        :raises SPSDKError: General error with write of Shadow register.
        """

        def write_reg(base_address: int, reg: RegsRegister) -> None:
            if not self.probe:
                raise SPSDKDebugProbeError(
                    "Shadow registers: Cannot use the communication function without defined debug probe."
                )

            if reg.width > 32:
                raise SPSDKError(
                    f"Invalid width ({reg.width}b) of shadow register ({reg.name}) to write to device."
                )
            # Create value
            value = reg.get_value()
            if reg.reverse:
                value = int.from_bytes(change_endianness(value_to_bytes(value)), "big")
            # Create verify mask
            verify_mask = 0
            if verify:
                bitfields = reg.get_bitfields()
                if bitfields:
                    for bitfield in bitfields:
                        verify_mask = verify_mask | (((1 << bitfield.width) - 1) << bitfield.offset)
                else:
                    verify_mask = (1 << reg.width) - 1

            self._write_shadow_reg(
                addr=base_address + reg.offset, data=value, verify_mask=verify_mask
            )

        try:
            reg = self.regs.find_reg(reg_name, include_group_regs=True)
            reg.set_value(data, raw=True)
            if reg.has_group_registers():
                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
                    write_reg(self.offset, sub_reg)
            else:
                write_reg(self.offset, reg=reg)
            # execute flash function handler if defined for a platfrom
            self.flush_func_handler()
        except SPSDKError as exc:
            raise SPSDKError(f"The set shadow register failed({str(exc)}).") from exc

    def get_register(self, reg_name: str) -> bytes:
        """The function returns value of the requested register.

        param reg: The register name.
        return: The value of requested register in bytes
        raises SPSDKDebugProbeError: The debug probe is not specified.
        """

        def read_reg(base_address: int, reg: RegsRegister) -> bytes:
            if not self.probe:
                raise SPSDKDebugProbeError(
                    "Shadow registers: Cannot use the communication function without defined debug probe."
                )
            if reg.width > 32:
                raise SPSDKError(
                    f"Invalid width ({reg.width}b) of shadow register ({reg.name}) to read from device."
                )
            read_value = self.probe.mem_reg_read(base_address + reg.offset).to_bytes(4, "big")
            if reg.reverse:
                read_value = change_endianness(read_value)
            return read_value

        try:
            reg = self.regs.find_reg(reg_name, include_group_regs=True)

            ret = bytearray()
            if reg.has_group_registers():
                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
                    ret.extend(read_reg(self.offset, sub_reg))
            else:
                ret.extend(read_reg(self.offset, reg=reg))
            return ret

        except SPSDKError as exc:
            raise SPSDKError(f"The get shadow register failed({str(exc)}).") from exc

    def create_fuse_blhost_script(self, reg_list: List[str]) -> str:
        """The function creates the BLHOST script to burn fuses.

        :param reg_list: The list of register to be burned.
        :raises SPSDKError: Exception in case of not existing register.
        :return: Content of BLHOST script file.
        """

        def add_reg(reg: RegsRegister) -> str:
            otp_index = reg.otp_index
            assert otp_index
            otp_value = "0x" + reg.get_bytes_value().hex()
            burn_fuse = f"# Fuse {reg.name}, index {otp_index} and value: {otp_value}.\n"
            burn_fuse += f"efuse-program-once {hex(otp_index)} {otp_value}\n"
            return burn_fuse

        ret = (
            "# BLHOST fuses programming script\n"
            f"# Generated by SPSDK {__version__}\n"
            f"# Chip: {self.device} rev:{self.revision}\n\n\n"
        )
        # Update list by antipole opposites registers
        for ap_reg_src, ap_reg_dst in self.config.get_antipole_regs(self.device).items():
            if ap_reg_src in reg_list:
                reg_list.insert(reg_list.index(ap_reg_src) + 1, ap_reg_dst)
        self.fuse_mode = True
        for reg_name in reg_list:
            try:
                reg = self.regs.find_reg(reg_name, True)
                # do recalculation based on fuse mode sets to ON!
                reg.set_value(reg.get_value())
            except SPSDKRegsErrorRegisterNotFound as exc:
                self.fuse_mode = False
                raise SPSDKError(
                    f"Register {reg_name} has not found for {self.device} device."
                ) from exc

            if reg.has_group_registers():
                for sub_reg in reg.sub_regs[:: -1 if reg.reverse_subregs_order else 1]:
                    ret += add_reg(sub_reg)
            else:
                ret += add_reg(reg)

        self.fuse_mode = False
        return ret

    def create_yaml_config(self, file_name: str, raw: bool = False, diff: bool = False) -> None:
        """The function creates the configuration YML file.

        :param file_name: The file_name (without extension) of stored configuration.
        :param raw: Raw output of configuration (including computed fields and anti-pole registers)
        :param diff: Get only configuration with difference value to reset state.
        """
        antipole_regs = None if raw else list(self.config.get_antipole_regs(self.device).values())
        computed_fields = None if raw else self.config.get_computed_fields(self.device)

        yaml = YAML()
        yaml.indent(sequence=SPSDK_YML_INDENT * 2, offset=SPSDK_YML_INDENT)
        data = CM()

        description = CM()
        description.yaml_set_start_comment(
            f"NXP {self.device.upper()} Shadow registers configuration", indent=2
        )
        description.insert(1, "device", self.device, comment="The NXP device name.")
        description.insert(
            2, "version", __version__, comment="The SPSDK Shadow register tool version."
        )
        description.insert(3, "author", __author__, comment="The author of the configuration.")
        description.insert(4, "release", __release__, comment="The SPSDK release.")

        data["description"] = description
        data["registers"] = self.regs.create_yml_config(
            exclude_regs=antipole_regs,
            exclude_fields=computed_fields,
            indent=2,
            diff=diff,
        )
        write_file(ConfigTemplate.convert_cm_to_yaml(data), file_name, encoding="utf8")

    def load_yaml_config(self, file_name: str, raw: bool = False) -> None:
        """The function loads the configuration from YML file.

        :param file_name: The file_name (without extension) of stored configuration.
        :param raw: Raw input of configuration (including computed fields and anti-pole registers)
        :raises SPSDKError: When the configuration file not found.
        """
        antipole_regs = None if raw else list(self.config.get_antipole_regs(self.device).values())
        computed_fields = None if raw else self.config.get_computed_fields(self.device)
        data = load_configuration(file_name)
        self.regs.load_yml_config(data["registers"], antipole_regs, computed_fields)
        if not raw:
            # Just update only configured registers
            exclude_hooks = list(set(self.regs.get_reg_names()) - set(data["registers"].keys()))
            self.regs.run_hooks(exclude_hooks)

        logger.debug("The shadow registers has been loaded from configuration.")

    @staticmethod
    def reg_antipolize_src_handler(val: int, context: Any) -> int:
        """Antipolize given register value.

        :param val: Input register value.
        :param context: The method context.
        :return: Antipolized value.
        """
        dst_reg: RegsRegister = context
        dst_reg.set_value(val ^ 0xFFFFFFFF, raw=True)
        return val

    @staticmethod
    def reg_antipolize_dst_handler(val: int, context: Any) -> int:
        """Keep same antipolized register value in computed register.

        :param val: Input register value.
        :param context: The method context.
        :return: Antipolized value.
        """
        src_reg: RegsRegister = context
        val = src_reg.get_value()
        new_val = val ^ 0xFFFFFFFF
        return new_val

    def flush_func_handler(self) -> None:
        """A function to determine and execute the flush-func handler.

        :param self: Input Value.
        :raises SPSDKError: Raises when the computing routine is not found.
        """
        flush_func = self.config.get_device_value("flush_func", device=self.device)
        if flush_func:
            if hasattr(self, flush_func):
                method_ref = getattr(self, flush_func)
                method_ref()
            else:
                raise SPSDKError(f"The '{flush_func}' function doesn't exists.")

    def reg_computed_fields_handler(self, val: bytes, context: Any) -> bytes:
        """Recalculate all fields for given register value.

        :param val: Input register value.
        :param context: The method context (fields).
        :return: recomputed value.
        :raises SPSDKError: Raises when the computing routine is not found.
        """
        fields: dict = context
        for method in fields.values():
            if hasattr(self, method):
                method_ref = getattr(self, method)
                val = method_ref(val)
            else:
                raise SPSDKError(f"The '{method}' compute function doesn't exists.")

        return val

    # CRC8 - ITU
    @staticmethod
    def crc_update(data: bytes, crc: int = 0, is_final: bool = True) -> int:
        """The function compute the CRC8 ITU method from given bytes.

        :param data: Input data to compute CRC.
        :param crc: The seed for CRC.
        :param is_final: The flag the the function should return final result.
        :return: The CRC result.
        """
        k = 0
        data_len = len(data)
        while data_len != 0:
            data_len -= 1
            carry = data[k]
            k += 1
            for i in range(8):
                bit = (crc & 0x80) != 0
                if (carry & (0x80 >> i)) != 0:
                    bit = not bit
                crc <<= 1
                if bit:
                    crc ^= 0x07
            crc &= 0xFF
        if is_final:
            return (crc & 0xFF) ^ 0x55
        return crc & 0xFF

    @staticmethod
    def comalg_dcfg_cc_socu_crc8(val: int) -> int:
        """Function that creates the crc for DCFG_CC_SOCU.

        :param val: Input DCFG_CC_SOCU Value.
        :return: Returns the value of DCFG_CC_SOCU with computed CRC8 field.
        """
        in_val = bytearray(3)
        for i in range(3):
            in_val[i] = (val >> (8 + i * 8)) & 0xFF
        val &= ~0xFF
        val |= ShadowRegisters.crc_update(in_val)
        return val

    def comalg_dcfg_cc_socu_test_en(self, val: int) -> int:
        """Function fill up the DCFG_CC_SOCU DEV_TEST_EN set to True to satisfy MCU needs.

        :param val: Input DCFG_CC_SOCU Value.
        :return: Returns the value of DCFG_CC_SOCU with optionally enabled test mode.
        """
        if self.fuse_mode:
            return val & ~0x80000000

        return val | 0x80000000

    def rw61x_update_scratch_reg(self) -> None:
        """Function updates scracth register for RW61x, This enables the shadow register functionality.

        :param self: Input Value.
        """
        addr = 0x5003B498
        value = 0xA7C56B9E
        self._write_shadow_reg(addr=addr, data=value, verify_mask=True)

    @staticmethod
    def comalg_do_nothing(val: int) -> int:
        """Function that do nothing.

        :param val: Input Value.
        :return: Returns same value as it get.
        """
        return val


def enable_debug(probe: DebugProbe, ap_mem: int = 0) -> bool:
    """Function that enables debug access ports on devices with debug mailbox.

    :param probe: Initialized debug probe.
    :param ap_mem: Index of Debug access port for memory interface.
    :return: True if debug port is enabled, False otherwise
    :raises SPSDKError: Unlock method failed.
    """
    debug_enabled = False
    try:
        logger.debug("step 3: Check if AHB is enabled")

        if not test_ahb_access(probe, ap_mem):
            logger.debug("Locked Device. Launching unlock sequence.")

            # Start debug mailbox system
            StartDebugSession(dm=DebugMailbox(debug_probe=probe)).run()

            # Recheck the AHB access
            if test_ahb_access(probe, ap_mem):
                logger.debug("Access granted")
                debug_enabled = True
            else:
                logger.debug("Enable debug operation failed!")
        else:
            logger.debug("Unlocked Device")
            debug_enabled = True

    except AttributeError as exc:
        raise SPSDKError(f"Invalid input parameters({str(exc)})") from exc

    except SPSDKDebugProbeError as exc:
        raise SPSDKError(f"Can't unlock device ({str(exc)})") from exc

    return debug_enabled



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2019-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""SPSDK version.

Having the version in a separate file makes it easier to share it with setup.py
"""

__version__ = "1.11.0"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2019-2022 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Connection options for SDP."""

from .base import Interface, SDPInterface
from .uart import Uart, scan_uart
from .usb import (  # type: ignore  # ignore problems due to OS specific implementation
    RawHid,
    scan_usb,
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2021-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Helper module for blhost application."""

import contextlib
from copy import deepcopy
from typing import Any, Callable, Iterator, Optional, Union

import click

from spsdk import SPSDKError
from spsdk.mboot.commands import (
    KeyProvUserKeyType,
    TrustProvKeyType,
    TrustProvOemKeyType,
    TrustProvWrappingKeyType,
)
from spsdk.utils.misc import value_to_int


class OemGenMasterShareHelp(click.Command):
    """Class for customized "usage" help line for oem_gen_master_share command."""

    def format_usage(self, ctx: Any, formatter: Any) -> None:
        """Customizes "usage" help line for oem_gen_master_share command."""
        click.echo("Usage: blhost trust-provisioning oem_gen_master_share [OPTIONS]")
        indent = 7 * "\t"
        click.echo(indent + "OEM_SHARE_INPUT_ADDR")
        click.echo(indent + "OEM_SHARE_INPUT_SIZE")
        click.echo(indent + "OEM_ENC_SHARE_OUTPUT_ADDR")
        click.echo(indent + "OEM_ENC_SHARE_OUTPUT_SIZE")
        click.echo(indent + "OEM_ENC_MASTER_SHARE_OUTPUT_ADDR")
        click.echo(indent + "OEM_ENC_MASTER_SHARE_OUTPUT_SIZE")
        click.echo(indent + "OEM_CUST_CERT_PUK_OUTPUT_ADDR")
        click.echo(indent + "OEM_CUST_CERT_PUK_OUTPUT_SIZE")


class OemSetMasterShareHelp(click.Command):
    """Class for customized "usage" help line for oem_set_master_share command."""

    def format_usage(self, ctx: Any, formatter: Any) -> None:
        """Customizes "usage" help line for oem_set_master_share command."""
        click.echo("Usage: blhost trust-provisioning oem_set_master_share [OPTIONS]")
        indent = 7 * "\t"
        click.echo(indent + "OEM_SHARE_INPUT_ADDR")
        click.echo(indent + "OEM_SHARE_INPUT_SIZE")
        click.echo(indent + "OEM_ENC_MASTER_SHARE_INPUT_ADDR")
        click.echo(indent + "OEM_ENC_MASTER_SHARE_INPUT_SIZE")


PROPERTIES_NAMES = {
    0: "list-properties",
    1: "current-version",
    2: "available-peripherals",
    3: "flash-start-address",
    4: "flash-size-in-bytes",
    5: "flash-sector-size",
    6: "flash-block-count",
    7: "available-commands",
    8: "check-status",
    9: "reserved",
    10: "verify-writes",
    11: "max-packet-size",
    12: "reserved-regions",
    13: "reserved_1",
    14: "ram-start-address",
    15: "ram-size-in-bytes",
    16: "system-device-id",
    17: "security-state",
    18: "unique-device-id",
    19: "flash-fac-support",
    20: "flash-access-segment-size",
    21: "flash-access-segment-count",
    22: "flash-read-margin",
    23: "qspi/otfad-init-status",
    24: "target-version",
    25: "external-memory-attributes",
    26: "reliable-update-status",
    27: "flash-page-size",
    28: "irq-notify-pin",
    29: "pfr-keystore_update-opt",
    30: "byte-write-timeout-ms",
    31: "fuse-locked-status",
}

KW45XX = {
    10: "verify-erase",
    20: "boot-status",
    21: "loadable-fw-version",
    22: "fuse-program-voltage",
}

PROPERTIES_OVERRIDE = {"kw45xx": KW45XX, "k32w1xx": KW45XX}


def parse_property_tag(property_tag: str, family: Optional[str] = None) -> int:
    """Convert the property as name or stringified number into integer.

    :param property_tag: Name or number of the property tag
    :param family: supported family
    :return: Property integer tag
    """
    properties_dict = deepcopy(PROPERTIES_NAMES)
    if family and family in PROPERTIES_OVERRIDE.keys():
        properties_dict.update(PROPERTIES_OVERRIDE[family])
    try:
        return value_to_int(property_tag)
    except SPSDKError:
        for key, value in properties_dict.items():
            if value == property_tag:
                return key
        return 0xFF


def parse_key_prov_key_type(key_type: str) -> int:
    """Convert the key type as name or stringified number into integer.

    :param key_type: Name or number of the Key type
    :return: key type number
    """
    return _parse_key_type(key_type, KeyProvUserKeyType, 0xFF)


def parse_trust_prov_oem_key_type(key_type: str) -> int:
    """Convert the key type as name or stringified number into integer.

    :param key_type: Name or number of the Key type
    :return: key type number
    """
    return _parse_key_type(key_type, TrustProvOemKeyType)


def parse_trust_prov_key_type(key_type: str) -> int:
    """Convert the key type as name or stringified number into integer.

    :param key_type: Name or number of the Key type
    :return: key type number
    """
    return _parse_key_type(key_type, TrustProvKeyType)


def parse_trust_prov_wrapping_key_type(key_type: str) -> int:
    """Convert the key type as name or stringified number into integer.

    :param key_type: Name or number of the Key type
    :return: key type number
    """
    return _parse_key_type(key_type, TrustProvWrappingKeyType)


def _parse_key_type(user_input: str, collection: Any, default: Optional[int] = None) -> int:
    try:
        return value_to_int(user_input)
    except SPSDKError:
        key_type = user_input.upper()
        key_type_int = collection.get(key_type, default)
        if key_type_int is None:
            raise SPSDKError(  # pylint: disable=raise-missing-from
                f"Unable to find '{user_input}' in '{collection.__name__}'"
            )
        return key_type_int


@contextlib.contextmanager
def progress_bar(
    suppress: bool = False, **progress_bar_params: Union[str, int]
) -> Iterator[Callable[[int, int], None]]:
    """Creates a progress bar and return callback function for updating the progress bar.

    :param suppress: Suppress the progress bar creation; return an empty callback, defaults to False
    :param progress_bar_params: Standard parameters for click.progressbar
    :yield: Callback for updating the progress bar
    """
    if suppress:
        yield lambda _x, _y: None
    else:
        with click.progressbar(length=100, **progress_bar_params) as p_bar:  # type: ignore

            def progress(step: int, total_steps: int) -> None:
                per_step = 100 / total_steps
                increment = step * per_step - p_bar.pos
                p_bar.update(round(increment))

            yield progress



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2021-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Trust provisioning host adapters interfaces."""
from typing import Any, Dict, List, Optional, Type, Union

from .data_container.audit_log import AuditLog
from .exceptions import SPSDKTpError


class TpIntfDescription:
    """Description of TP interface."""

    def __init__(
        self,
        name: str,
        intf: Optional[Union[Type["TpDevInterface"], Type["TpTargetInterface"]]],
        description: str,
        settings: Optional[Dict],
        version: str = "0.0.0",
    ) -> None:
        """Root Of Trust device description.

        :param name: Name of the device.
        :param description: Description of the device.
        :param settings: Dictionary of settings needed to properly setup the device.
        """
        self.name = name
        self.intf = intf
        self.description = description
        self.settings = settings
        self.version = version

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}: id={self.get_id()}, version={self.version}>"

    def as_dict(self) -> Dict[str, Any]:
        """Returns whole record as dictionary.

        :return: All variables of class in dictionary.
        """
        dictionary = vars(self).copy()
        dictionary.pop("settings")  # The settings are not important in this case
        dictionary.pop("intf")  # The interface is not important in this case
        return dictionary

    def get_id(self) -> Union[str, int]:
        """Returns the ID of the interface."""
        raise NotImplementedError()

    def get_id_hash(self) -> str:
        """Return the ID hash of the interface."""
        raise NotImplementedError()

    def create_interface(self) -> Union["TpDevInterface", "TpTargetInterface"]:
        """Return TP Device or Target associated with this descriptor."""
        assert self.intf
        return self.intf(self)


class TpInterface:
    """Generic TP Interface."""

    NAME = "Interface"

    @classmethod
    def get_connected_interfaces(cls, settings: Optional[Dict] = None) -> List[TpIntfDescription]:
        """Get all connected TP devices of this adapter.

        :param settings: Possible settings to determine the way to find connected device, defaults to None.
        :return: List of all founded TP devices.
        """
        raise NotImplementedError()

    def __init__(self, descriptor: TpIntfDescription) -> None:
        """Initialization of TP device.

        :param descriptor: Device Interface descriptor.
        """
        self.descriptor = descriptor

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}: desc={repr(self.descriptor)}>"

    def open(self) -> None:
        """Open the TP interface adapter.

        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    @property
    def is_open(self) -> bool:
        """Check if the TP interface is open."""
        return True

    def close(self) -> None:
        """Close the TP interface adapter.

        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    @staticmethod
    def get_help() -> str:
        """Return help for this interface, including settings description.

        :return: String with help.
        """
        raise NotImplementedError()

    @classmethod
    def get_validation_schemas(cls) -> List[Dict[str, Any]]:
        """Return all additional validation schemas for interface.

        return: List of all additional validation schemas.
        """
        raise NotImplementedError()


class TpDevInterface(TpInterface):
    """Trust provisioning - Trust provisioning device adapter interface."""

    get_connected_devices = TpInterface.get_connected_interfaces

    def __init__(self, descriptor: TpIntfDescription) -> None:
        """Initialization of TP device."""
        super().__init__(descriptor=descriptor)

    def get_challenge(self, timeout: Optional[int] = None) -> bytes:
        """Request challenge from the TP device.

        :param timeout: Timeout of operation in milliseconds.
        :return: Challenge record, to be used for TP communication.
        """
        raise NotImplementedError()

    def authenticate_response(self, tp_data: bytes, timeout: Optional[int] = None) -> bytes:
        """Request TP device for TP authentication of connected MCU.

        :param tp_data: TP response of connected MCU.
        :param timeout: Timeout of operation in milliseconds.
        :return: Wrapped data after TP response processing.
        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    def upload(self, config_data: dict, config_dir: Optional[str] = None) -> None:
        """Upload the user data into the provisioning device."""
        raise NotImplementedError()

    def setup(self) -> None:
        """Setup the provisioning device."""
        # By default do nothing

    def seal(self) -> None:
        """Seal the provisioning device (advance its lifecycle)."""
        raise NotImplementedError()

    def get_prov_counter(self) -> int:
        """Get actual provisioning counter."""
        raise NotImplementedError()

    def get_prov_remainder(self) -> int:
        """Get the number of remaining provisioning attempts."""
        raise NotImplementedError()

    def check_log_owner(self, audit_log: str) -> None:
        """Check if this TP Device's ID is present in the audit log."""
        my_id = str(self.descriptor.get_id())
        audit_log_properties = AuditLog.properties(audit_log)
        if my_id != audit_log_properties.tp_device_id:
            raise SPSDKTpError(
                "Audit log ID doesn't match. "
                f"TP Device ID: {my_id}, Audit log ID: {audit_log_properties.tp_device_id}"
            )


class TpTargetInterface(TpInterface):
    """Trust provisioning - provisioned target adapter interface."""

    get_connected_targets = TpInterface.get_connected_interfaces

    def __init__(self, descriptor: TpIntfDescription) -> None:
        """Initialization of TP target.

        :param descriptor: Target Interface descriptor.
        """
        super().__init__(descriptor=descriptor)

    @property
    def uses_uart(self) -> bool:
        """Check if the adapter is using UART for communication."""
        return False

    @property
    def uses_usb(self) -> bool:
        """Check if the adapter is using USB for communication."""
        return False

    def reset_device(self) -> None:
        """Reset the connected provisioned device.

        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    def load_sb_file(self, sb_file: bytes, timeout: Optional[int] = None) -> None:
        """Load SB file into provisioned device.

        :param sb_file: SB file data to be loaded into provisioned device.
        :param timeout: Timeout of operation in milliseconds.
        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    def prove_genuinity_challenge(self, challenge: bytes, timeout: Optional[int] = None) -> bytes:
        """Prove genuinity and get back the TP response to continue process of TP.

        :param challenge: Challenge data to start TP process.
        :param timeout: Timeout of operation in milliseconds.
        :return: Trust provisioning response for TP process.
        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    def set_wrapped_data(self, wrapped_data: bytes, timeout: Optional[int] = None) -> None:
        """Provide wrapped data to provisioned device.

        :param wrapped_data: Wrapped data to finish TP process.
        :param timeout: Timeout of operation in milliseconds.
        raises NotImplementedError: The function is not implemented.
        """
        raise NotImplementedError()

    def read_memory(self, address: int, length: int, memory_id: int = 0) -> bytes:
        """Read data from the target's memory.

        :param address: Start address
        :param length: Number of bytes to read
        :param memory_id: Memory ID, defaults to 0
        :raises NotImplementedError: This function is not implemented
        :return: Data read from the target
        """
        raise NotImplementedError()

    def write_memory(self, address: int, data: bytes, memory_id: int = 0) -> None:
        """Write data to the target's memory.

        :param address: Start address
        :param data: Data to write
        :param memory_id: Memory ID, defaults to 0
        :raises NotImplementedError: This function is not implemented
        """
        raise NotImplementedError()

    def erase_memory(self, address: int, length: int, memory_id: int = 0) -> None:
        """Erase target's memory.

        :param address: Start address
        :param length: Number of bytes to erase
        :param memory_id: Memory ID, defaults to 0
        :raises NotImplementedError: This function is not implemented
        """
        raise NotImplementedError()

    # pylint: disable=no-self-use    # inherited classes may use self
    def check_provisioning_firmware(self) -> bool:
        """Check whether the Provisioning Firmware booted properly."""
        return True



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause
"""SPSDK plugins manager."""

import logging
import os
import sys
from importlib.machinery import ModuleSpec
from importlib.util import find_spec, module_from_spec, spec_from_file_location
from types import ModuleType
from typing import Dict, List, Optional

import importlib_metadata

from spsdk.exceptions import SPSDKError, SPSDKTypeError
from spsdk.utils.easy_enum import Enum
from spsdk.utils.misc import SingletonMeta

logger = logging.getLogger(__name__)


class PluginType(Enum):
    """Contains commands tags."""

    SIGNATURE_PROVIDER = (0, "spsdk.sp", "Signature provider")


class PluginsManager(metaclass=SingletonMeta):
    """Plugin manager."""

    def __init__(self) -> None:
        """Plugin manager constructor."""
        self.plugins: Dict[str, ModuleType] = {}

    def load_from_entrypoints(self, group_name: Optional[str] = None) -> int:
        """Load modules from given setuptools group.

        :param group_name: Entry point group to load plugins

        :return: The number of loaded plugins.
        """
        if group_name is not None and not isinstance(group_name, str):
            raise SPSDKTypeError("Group name must be of string type.")
        group_names = (
            [group_name]
            if group_name is not None
            else [PluginType.name(tag) for tag in PluginType.tags()]
        )

        entry_points: List[importlib_metadata.EntryPoint] = []
        for group_name in group_names:
            eps = importlib_metadata.entry_points(group=group_name)
            entry_points.extend(eps)

        count = 0
        for ep in entry_points:
            try:
                plugin = ep.load()
            except ModuleNotFoundError:
                logger.warning(f"Module {ep.module} could not be loaded")
                continue
            logger.info(f"Plugin {ep.name} has been loaded.")
            self.register(plugin)
            count += 1
        return count

    def load_from_source_file(self, source_file: str) -> None:
        """Import Python source file directly.

        :param source_file: Path to python source file: absolute or relative to cwd
        :raises SPSDKError: If importing of source file failed
        """
        module_name = os.path.splitext(os.path.basename(source_file))[0]
        spec = spec_from_file_location(name=module_name, location=source_file)
        if not spec:
            raise SPSDKError(
                f"Source '{source_file}' does not exist. Check if it is valid file path name"
            )

        module = self._import_module_spec(spec)
        self.register(module)

    def load_from_module_name(self, module_name: str) -> None:
        """Import Python module directly.

        :param module_name: Module name to be imported
        :raises SPSDKError: If importing of source file failed
        """
        spec = find_spec(name=module_name)
        if not spec:
            raise SPSDKError(
                f"Source '{module_name}' does not exist.Check if it is valid file module name"
            )
        module = self._import_module_spec(spec)
        self.register(module)

    def _import_module_spec(self, spec: ModuleSpec) -> ModuleType:
        """Import module from module specification.

        :param spec: Module specification
        :return: Imported module type
        """
        module = module_from_spec(spec)
        try:
            sys.modules[spec.name] = module
            spec.loader.exec_module(module)  # type: ignore
            logger.debug(f"A module spec {spec.name} has been loaded.")
        except Exception as e:
            raise SPSDKError(f"Failed to load module spec {spec.name}: {e}") from e
        return module

    def register(self, plugin: ModuleType) -> None:
        """Register a plugin with the given name.

        :param plugin: Plugin as a module
        """
        plugin_name = self.get_plugin_name(plugin)
        if plugin_name in self.plugins:
            logger.debug(f"Plugin {plugin_name} has been aready registered.")
            return
        self.plugins[plugin_name] = plugin
        logger.debug(f"A plugin {plugin_name} has been registered.")

    def get_plugin(self, name: str) -> Optional[ModuleType]:
        """Return a plugin for the given name.

        :param name: Plugin name
        :return: Plugin or None if plugin with name is not registered
        """
        return self.plugins.get(name)

    def get_plugin_name(self, plugin: ModuleType) -> str:
        """Get canonical name of plugin.

        :param plugin: Plugin as a module
        :return: String with plugin name
        """
        name = getattr(plugin, "__name__", None)
        if name is None:
            raise SPSDKError("Plugin name could not be determined.")
        return name


def load_plugin_from_source(source: str) -> None:
    """Load plugin from source.

    :param source: The source to be loaded
        Accepted values:
            - Path to source file
            - Existing module name
            - Existing entrypoint
    """
    manager = PluginsManager()
    try:
        return manager.load_from_source_file(source)
    except SPSDKError:
        pass
    try:
        manager.load_from_module_name(source)
        return
    except SPSDKError:
        pass
    try:
        manager.load_from_entrypoints(source)
        return
    except SPSDKError:
        pass
    raise SPSDKError(f"Unable to load from source '{source}'.")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2022-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""AHAB abstract classes."""

from abc import ABC
from struct import calcsize, unpack
from typing import Tuple

from spsdk.exceptions import SPSDKLengthError, SPSDKParsingError, SPSDKValueError
from spsdk.utils.misc import check_range

LITTLE_ENDIAN = "<"
UINT8 = "B"
UINT16 = "H"
UINT32 = "L"
UINT64 = "Q"
RESERVED = 0


class Container(ABC):
    """Base class for any container."""

    @classmethod
    def fixed_length(cls) -> int:
        """Returns the length of a container which is fixed.

        i.e. part of a container holds fixed values, whereas some entries have
        variable length.
        """
        return calcsize(cls._format())

    def __len__(self) -> int:
        """Returns the total length of a container.

        The length includes the fixed as well as the variable length part.
        """
        return self.fixed_length()

    @classmethod
    def _format(cls) -> str:
        """Returns the container data format as defined by struct package.

        The base returns only endianness (LITTLE_ENDIAN).
        """
        return LITTLE_ENDIAN

    def export(self) -> bytes:
        """Exports container to final bytes array.

        :return: Bytes representation of container object.
        """
        raise NotImplementedError

    @classmethod
    def _check_fixed_input_length(cls, binary: bytes) -> None:
        """Checks the data length and container fixed length.

        This is just a helper function used throughout the code.

        :param Binary: Binary input data.
        :raises SPSDKLengthError: If containers length is larger than data length.
        """
        data_len = len(binary)
        fixed_input_len = cls.fixed_length()
        if data_len < fixed_input_len:
            raise SPSDKLengthError(
                f"Parsing error in fixed part of {cls.__name__} data!\n"
                f"Input data must be at least {fixed_input_len} bytes!"
            )


class HeaderContainer(Container):
    """A container with first byte defined as header - tag, length and version.

    Every "container" in AHAB consists of a header - tag, length and version.

    The only exception is the 'image array' or 'image array entry' respectively
    which has no header at all and SRK record, which has 'signing algorithm'
    instead of version. But this can be considered as a sort of SRK record
    'version'.
    """

    TAG = 0x00
    VERSION = 0x00

    def __init__(self, tag: int, length: int, version: int):
        """Class object initialized.

        :param tag: container tag.
        :param length: container length.
        :param version: container version.
        """
        self.length = length
        self.tag = tag
        self.version = version

    def __eq__(self, other: object) -> bool:
        if isinstance(other, (HeaderContainer, HeaderContainerInversed)):
            if (
                self.tag == other.tag
                and self.length == other.length
                and self.version == other.version
            ):
                return True

        return False

    @classmethod
    def _format(cls) -> str:
        return super()._format() + UINT8 + UINT16 + UINT8

    def validate_header(self) -> None:
        """Validates the header of container properties...

        i.e. tag e <0; 255>, otherwise an exception is raised.
        :raises SPSDKValueError: Any MAndatory field has invalid value.
        """
        if self.tag is None or not check_range(self.tag, end=0xFF):
            raise SPSDKValueError(f"AHAB: Head of Container: Invalid TAG Value: {self.tag}")
        if self.length is None or not check_range(self.length, end=0xFFFF):
            raise SPSDKValueError(f"AHAB: Head of Container: Invalid Length Value: {self.length}")
        if self.version is None or not check_range(self.version, end=0xFF):
            raise SPSDKValueError(f"AHAB: Head of Container: Invalid Version Value: {self.version}")

    @classmethod
    def parse_head(cls, binary: bytes) -> Tuple[int, int, int]:
        """Parse binary data to get head members.

        :param binary: Binary data.
        :raises SPSDKLengthError: Binary data length is not enough.
        :return: Tuple with TAG, LENGTH, VERSION
        """
        if len(binary) < 4:
            raise SPSDKLengthError(
                f"Parsing error in {cls.__name__} container head data!\n"
                "Input data must be at least 4 bytes!"
            )
        (version, length, tag) = unpack(HeaderContainer._format(), binary)
        return tag, length, version

    @classmethod
    def _check_container_head(cls, binary: bytes) -> None:
        """Compares the data length and container length.

        This is just a helper function used throughout the code.

        :param Binary: Binary input data.
        :raises SPSDKLengthError: If containers length is larger than data length.
        :raises SPSDKParsingError: If containers header value doesn't match.
        """
        cls._check_fixed_input_length(binary)
        data_len = len(binary)
        (tag, length, version) = cls.parse_head(binary[: HeaderContainer.fixed_length()])

        if (
            isinstance(cls.TAG, int)
            and tag != cls.TAG
            or isinstance(cls.TAG, list)
            and not tag in cls.TAG
        ):
            raise SPSDKParsingError(
                f"Parsing error of {cls.__name__} data!\n"
                f"Invalid TAG 0x{hex(tag)} loaded, expected 0x{hex(cls.TAG)}!"
            )

        if data_len < length:
            raise SPSDKLengthError(
                f"Parsing error of {cls.__name__} data!\n"
                f"At least {length} bytes expected, got {data_len} bytes!"
            )

        if (
            isinstance(cls.VERSION, int)
            and version != cls.VERSION
            or isinstance(cls.VERSION, list)
            and not version in cls.VERSION
        ):
            raise SPSDKParsingError(
                f"Parsing error of {cls.__name__} data!\n"
                f"Invalid VERSION {version} loaded, expected {cls.VERSION}!"
            )


class HeaderContainerInversed(HeaderContainer):
    """A container with first byte defined as header - tag, length and version.

    It same as "HeaderContainer" only the tag/length/version are in reverse order in binary form.
    """

    @classmethod
    def parse_head(cls, binary: bytes) -> Tuple[int, int, int]:
        """Parse binary data to get head members.

        :param binary: Binary data.
        :raises SPSDKLengthError: Binary data length is not enough.
        :return: Tuple with TAG, LENGTH, VERSION
        """
        if len(binary) < 4:
            raise SPSDKLengthError(
                f"Parsing error in {cls.__name__} container head data!\n"
                "Input data must be at least 4 bytes!"
            )
        # Only SRK Table has splitted tag and version in binary format
        (tag, length, version) = unpack(HeaderContainer._format(), binary)
        return tag, length, version



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2019-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""Module for functionality shared across all interfaces."""

from abc import ABC, abstractmethod
from typing import Any, Optional


class SDPInterface(ABC):
    """Base class for all Interface classes."""

    expect_status = True

    @property
    def is_opened(self) -> bool:
        """Indicates whether interface is open."""
        return False

    @abstractmethod
    def open(self) -> None:
        """Open the interface."""

    @abstractmethod
    def close(self) -> None:
        """Close the interface."""

    @abstractmethod
    def read(self, length: Optional[int] = None) -> Any:
        """Read data from the device."""

    @abstractmethod
    def write(self, packet: Any) -> None:
        """Write a packet to the device."""

    @abstractmethod
    def conf(self, config: dict) -> None:
        """Configure device."""

    @abstractmethod
    def info(self) -> str:
        """Return string containing information about the interface."""


# for backwards compatibility
Interface = SDPInterface



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2020-2023 NXP
#
# SPDX-License-Identifier: BSD-3-Clause

"""SignatureProvider is an Interface for all potential signature providers.

Each concrete signature provider needs to implement:
- sign(data: bytes) -> bytes
- into() -> str
"""

import abc
import getpass
import logging
import math
from types import ModuleType
from typing import Any, Dict, List, Optional, Type, Union

from Crypto.PublicKey import ECC

from spsdk import crypto
from spsdk.exceptions import SPSDKError, SPSDKUnsupportedOperation, SPSDKValueError
from spsdk.utils.misc import find_file
from spsdk.utils.plugins import PluginsManager, PluginType

logger = logging.getLogger(__name__)


class SignatureProvider(abc.ABC):
    """Abstract class (Interface) for all signature providers."""

    # Subclasses override the following signature provider type
    sp_type = "INVALID"

    @abc.abstractmethod
    def sign(self, data: bytes) -> bytes:
        """Return signature for data."""

    @property
    @abc.abstractmethod
    def signature_length(self) -> int:
        """Return length of the signature."""

    def verify_public_key(self, public_key: bytes) -> bool:
        """Verify if given public key matches private key."""
        raise SPSDKUnsupportedOperation("Verify method is not supported.")

    def info(self) -> str:
        """Provide information about the Signature provider."""
        return self.__class__.__name__

    @staticmethod
    def convert_params(params: str) -> Dict[str, str]:
        """Coverts creation params from string into dictionary.

        e.g.: "type=file;file_path=some_path" -> {'type': 'file', 'file_path': 'some_path'}
        """
        try:
            result = dict([tuple(p.split("=")) for p in params.split(";")])  # type: ignore  #oh dear Mypy
        except ValueError as e:
            raise SPSDKValueError(
                "Parameter must meet following pattern: type=file;file_path=some_path"
            ) from e
        return result

    @classmethod
    def get_types(cls) -> List[str]:
        """Returns a list of all available signature provider types."""
        return [sub_class.sp_type for sub_class in cls.__subclasses__()]

    @classmethod
    def create(cls, params: Union[str, dict]) -> Optional["SignatureProvider"]:
        """Creates an concrete instance of signature provider."""
        load_plugins()
        if isinstance(params, str):
            params = cls.convert_params(params)
        sp_classes = cls.get_all_signature_providers()
        for klass in sp_classes:  # pragma: no branch  # there always be at least one subclass
            if klass.sp_type == params["type"]:
                del params["type"]
                unused_params = set(params) - set(klass.__init__.__code__.co_varnames)
                for unused_param in unused_params:
                    if unused_param != "search_paths":
                        logger.warning(
                            f"Removing unused parameter for {klass.sp_type} signature provider: {unused_param}"
                        )
                    del params[unused_param]
                return klass(**params)  # type: ignore  #oh dear Mypy
        logger.info(f"Signature provider of type {params['type']} was not found.")
        return None

    @staticmethod
    def get_all_signature_providers() -> List[Type["SignatureProvider"]]:
        """Get list of all available signature providers."""

        def get_subclasses(
            base_class: Type,
        ) -> List[Type["SignatureProvider"]]:
            """Recursively find all subclasses."""
            subclasses = []
            for subclass in base_class.__subclasses__():
                subclasses.append(subclass)
                subclasses.extend(get_subclasses(subclass))
            return subclasses

        return get_subclasses(SignatureProvider)


class PlainFileSP(SignatureProvider):
    """PlainFileSP is a SignatureProvider implementation that uses plain local files."""

    sp_type = "file"

    def __init__(
        self,
        file_path: str,
        password: Optional[str] = None,
        encoding: str = "PEM",  # pylint: disable=unused-argument
        hash_alg: Optional[str] = None,
        search_paths: Optional[List[str]] = None,
        mode: Optional[str] = None,
    ) -> None:
        """Initialize the plain file signature provider.

        :param file_path: Path to private file
        :param password: Password in case of encrypted private file, defaults to None
        :param encoding: Private file encoding, defaults to 'PEM'
        :param hash_alg: Hash for the signature, defaults to 'sha256'
        :param search_paths: List of paths where to search for the file, defaults to None
        :param mode: Optionally there could be specified mode of signature algorithm.
            For example to switch EC signature to deterministic mode 'deterministic-rfc6979' must be used
        :raises SPSDKError: Invalid Private Key
        """
        self.file_path = find_file(file_path=file_path, search_paths=search_paths)
        password_bytes = str.encode(password) if password is not None else None
        self.private_key = crypto.load_private_key(self.file_path, password=password_bytes)
        assert isinstance(self.private_key, crypto._PrivateKeyTuple)
        if hash_alg:
            hash_alg_name = hash_alg
        else:
            if isinstance(self.private_key, crypto.RSAPrivateKey):
                hash_alg_name = "sha256"

            elif isinstance(self.private_key, crypto.EllipticCurvePrivateKey):
                # key_size <= 256       =>  SHA256
                # 256 < key_size <= 384 =>  SHA384
                # 384 < key_size        =>  SHA512
                if self.private_key.key_size <= 256:
                    hash_size = 256
                elif 256 < self.private_key.key_size <= 384:
                    hash_size = 384
                else:
                    hash_size = 512
                hash_alg_name = f"sha{hash_size}"
            else:
                raise SPSDKError(
                    f"Unsupported private key by signature provider: {str(self.private_key)}"
                )
        self.mode = mode
        self.hash_alg = getattr(crypto.hashes, hash_alg_name.upper())()

    @property
    def signature_length(self) -> int:
        """Return length of the signature."""
        sig_len = math.ceil(self.private_key.key_size / 8)
        if isinstance(self.private_key, crypto.EllipticCurvePrivateKey):
            sig_len *= 2
        return sig_len

    def verify_public_key(self, public_key: bytes) -> bool:
        """Verify if given public key matches private key."""
        from spsdk.utils.crypto.common import crypto_backend, serialize_ecc_signature

        crypto_public_key = crypto.loaders.load_public_key_from_data(public_key)
        assert isinstance(crypto_public_key, crypto._PublicKeyTuple)
        data = bytes()
        if isinstance(crypto_public_key, crypto.RSAPublicKey):
            signature = self._rsa_sign(data)
            is_matching = crypto_backend().rsa_verify(
                pub_key_mod=crypto_public_key.public_numbers().n,
                pub_key_exp=crypto_public_key.public_numbers().e,
                signature=signature,
                data=data,
            )
            return is_matching
        else:  # public_key can be only one of RSAPublicKey | EllipticCurvePublicKey type
            signature = self._ecc_sign(data)
            signature_data = serialize_ecc_signature(signature, self.signature_length // 2)
            is_matching = crypto_backend().ecc_verify(
                public_key=crypto_public_key, signature=signature_data, data=data  # type: ignore
            )
            return is_matching

    def info(self) -> str:
        """Return basic into about the signature provider."""
        msg = super().info()
        msg += f"\nKey path: {self.file_path}\n"
        return msg

    def sign(self, data: bytes) -> bytes:
        """Return the signature for data."""
        if isinstance(self.private_key, crypto.RSAPrivateKey):
            return self._rsa_sign(data)
        else:  # self.private_key can be only one of RSAPrivateKey | RSAPublicKey type
            return self._ecc_sign(data)

    def _rsa_sign(self, data: bytes) -> bytes:
        """Return RSA signature."""
        assert isinstance(self.private_key, crypto.RSAPrivateKey)
        signature = self.private_key.sign(
            data=data, padding=crypto.padding.PKCS1v15(), algorithm=self.hash_alg
        )
        return signature

    def _ecc_sign(self, data: bytes) -> bytes:
        """Return ECC signature."""
        assert isinstance(self.private_key, crypto.EllipticCurvePrivateKey)
        if self.mode and self.mode == "deterministic-rfc6979":
            from spsdk.utils.crypto.backend_internal import internal_backend

            private_key_bytes = self.private_key.private_bytes(
                encoding=crypto.Encoding.PEM,
                format=crypto.serialization.PrivateFormat.PKCS8,
                encryption_algorithm=crypto.serialization.NoEncryption(),
            )
            crypto_dome_key = ECC.import_key(private_key_bytes)
            signature = internal_backend.ecc_sign(crypto_dome_key, data)
        else:
            signature = self.private_key.sign(
                data=data, signature_algorithm=crypto.ec.ECDSA(self.hash_alg)
            )
        return signature


class InteractivePlainFileSP(PlainFileSP):
    """SignatureProvider implementation that uses plain local file in an "interactive" mode.

    If the private key is encrypted, the useer will be promped for password
    """

    sp_type = "interactive_file"

    def __init__(
        self,
        file_path: str,
        encoding: str = "PEM",
        hash_alg: Optional[str] = None,
        search_paths: Optional[List[str]] = None,
        mode: Optional[str] = None,
    ) -> None:
        """Initialize the interactive plain file signature provider.

        :param file_path: Path to private file
        :param encoding: Private file encoding, defaults to 'PEM'
        :param hash_alg: Hash for the signature, defaults to 'sha256'
        :param search_paths: List of paths where to search for the file, defaults to None
        :param mode: Optionally there could be specified mode of signature algorithm.
            For example to switch EC signature to deterministic mode 'deterministic-rfc6979' must be used
        :raises SPSDKError: Invalid Private Key
        """
        password = None
        try:
            self.file_path = find_file(file_path=file_path, search_paths=search_paths)
            self.private_key = crypto.load_private_key(self.file_path)
        except TypeError:
            password = getpass.getpass(
                prompt="Private key is encrypted.Enter password: ", stream=None
            )
        super().__init__(file_path, password, encoding, hash_alg, search_paths, mode)


def get_signature_provider(
    sp_cfg: Optional[str] = None, local_file_key: Optional[str] = None, **kwargs: Any
) -> SignatureProvider:
    """Get the signature provider from configuration.

    :param sp_cfg: Configuration of signature provider.
    :param local_file_key: Optional backward compatibility
        option to specify just path to local private key.
    :param kwargs: Additional parameters, that could be accepted by Signature providers.
    :return: Signature Provider instance.
    :raises SPSDKError: Invalid input configuration.
    """
    if sp_cfg:
        params: Dict[str, Union[str, List[str]]] = {}
        params.update(SignatureProvider.convert_params(sp_cfg))
        for k, v in kwargs.items():
            if not k in params:
                params[k] = v
        signature_provider = SignatureProvider.create(params=params)
    elif local_file_key:
        signature_provider = PlainFileSP(
            file_path=local_file_key,
            search_paths=kwargs.get("search_paths"),
            mode=kwargs.get("mode"),
        )
    else:
        raise SPSDKValueError("No signature provider configuration is provided")

    if not signature_provider:
        raise SPSDKError(f"Cannot create signature provider from: {sp_cfg or local_file_key}")

    return signature_provider


def load_plugins() -> Dict[str, ModuleType]:
    """Load all installed signature provider plugins."""
    plugins_manager = PluginsManager()
    plugins_manager.load_from_entrypoints(PluginType.name(PluginType.SIGNATURE_PROVIDER))
    return plugins_manager.plugins
